#include "HtmlSyntaxColorer.h"
#include <stdio.h>


extern "C" __declspec(dllexport) SyntaxColorer *instantiate_language();

SyntaxColorer *instantiate_language()
{
	return(new HtmlSyntaxColorer());
}

extern "C" __declspec(dllexport) bool handles_type(BString mimetype);

bool handles_type(BString mimetype)
{
	if(mimetype.ICompare("text/html")==0)
		return(true);
	else
		return(false);
}



HtmlSyntaxColorer::HtmlSyntaxColorer()
{
}

HtmlSyntaxColorer::~HtmlSyntaxColorer()
{
}

bool	HtmlSyntaxColorer::HandlesType(const char* Mimetype)
{
	BString	temp(Mimetype);
	if(temp.Compare("text/HTML")==0)
		return(true);
	else
		return(false);
}

void	HtmlSyntaxColorer::InitColorGet()
{
	m_ListPos=0;
}

bool	HtmlSyntaxColorer::GetColorRange(int &Start,int &Stop,rgb_color &Color)
{
	TextBlock*	Block;

	while(m_ListPos<m_TextList.CountItems())
	{
		if((Block=(TextBlock*)m_TextList.ItemAt(m_ListPos))->m_Dirty==true)
		{
			Start=Block->m_Start;
			Stop=Block->m_Stop;
			switch(Block->m_Type)
			{
				case TEXT:	
					Color.red=0;	
					Color.green=0;
					Color.blue=0;
				break;
				case TAG:
					Color.red=0;
					Color.green=0;
					Color.blue=255;
				break;
				case ERROR:
					Color.red=255;
					Color.green=0;
					Color.blue=0;
				break;
				case COMMENT:
					Color.red=100;
					Color.green=100;
					Color.blue=100;
				break;
			}
			Block->m_Dirty=false;
			return(true);
		}
		m_ListPos++;
	}
	return(false);
}

void	HtmlSyntaxColorer::ReparseAll(const char* Text)
{
	// Remove all elements from the list.
//	BStopWatch*	watch=new BStopWatch("Deletion Timer: ");
	void	*temp;
	while((temp=m_TextList.RemoveItem(0L))!=NULL)
		delete(temp);
		
//	delete(watch);
	
	m_Text=Text;
	
	m_Mode=TEXT;
	m_Offset=0;
//	watch=new BStopWatch("Parsing Timer: ");

	uint	len=strlen(m_Text);
	//printf("Text length: %d\n",len);	
	for(uint pos=0;pos<len;pos++)
	{
		if(m_Text[pos]=='<'&&m_Text[pos+1]=='!'&&m_Text[pos+2]=='-'&&m_Text[pos+3]=='-')
		{
			if(m_Mode!=TEXT)
			{
				if(pos>(uint)m_Offset)
					AddItem(m_Offset,pos,ERROR);
				m_Mode=COMMENT_BEGIN;
				m_Offset=pos;							
			}
			else
			{
				if(pos>(uint)m_Offset)
					AddItem(m_Offset,pos-1,TEXT);
				m_Mode=COMMENT_BEGIN;
				m_Offset=pos;							
			
			}			
		}
		else if(m_Text[pos]=='<')
		{
			if(m_Mode==TAG_BEGIN)
			{
				if(pos>(uint)m_Offset)
					AddItem(m_Offset,pos,ERROR);
				m_Mode=TAG_BEGIN;
				m_Offset=pos;
			}
			if(m_Mode==TEXT)
			{
				if(pos>(uint)m_Offset)
					AddItem(m_Offset,pos-1,TEXT);
				m_Mode=TAG_BEGIN;
				m_Offset=pos;
			}
		}
		
		if(m_Text[pos]=='>')
		{
			if(m_Mode==TAG_BEGIN)
			{
				if(pos>(uint)m_Offset)
					AddItem(m_Offset,pos,TAG);
				m_Mode=TEXT;
				m_Offset=pos+1;
			}		
		}
		
		if(pos>=2)	//Make sure we dont check an end comment
		{
			if(m_Text[pos]=='>'&&m_Text[pos-2]=='-'&&m_Text[pos-1]=='-')
			{
				if(m_Mode==COMMENT_BEGIN)
				{
					if(pos>=(uint)m_Offset)
						AddItem(m_Offset,pos,COMMENT);
					m_Mode=TEXT;
					m_Offset=pos+1;				
				}
			}
		}

	}
	
	switch(m_Mode)
	{
	case COMMENT_BEGIN:
		AddItem(m_Offset,len,COMMENT);
	break;
	case TEXT:
		AddItem(m_Offset,len,TEXT);
	break;
	default:
		AddItem(m_Offset,len,ERROR);
	break;	
	}
	
//	delete(watch);
}

void	HtmlSyntaxColorer::AddItem(int Start,int Stop,int Type)
{
	TextBlock*	block=new TextBlock;
	block->m_Start=Start;
	block->m_Stop=Stop;
	block->m_Type=Type;
	block->m_Dirty=true;
	m_TextList.AddItem(block);
}


void	HtmlSyntaxColorer::TextInserted(int Offset,int Length,const char* Text)
{
	TextBlock*	temp;
	ReparseAll(Text);

	int	textlength=m_TextList.CountItems();

	for(int i=0;i<textlength;i++)
	{	
		temp=(TextBlock*)m_TextList.ItemAt(i);
		temp->m_Dirty=false;
	}

	for(int i=0;i<textlength;i++)
	{
		temp=(TextBlock*)m_TextList.ItemAt(i);
		if(Offset>=temp->m_Start&&(Length+Offset)<=(temp->m_Stop+1))
		{
			temp->m_Dirty=true;

			if(i<textlength-2)
			{
				temp=(TextBlock*)m_TextList.ItemAt(i+1);
				temp->m_Dirty=true;
				temp=(TextBlock*)m_TextList.ItemAt(i+2);
				temp->m_Dirty=true;
			}

			if(temp->m_Type==COMMENT)
				for(int b=i;b<textlength;b++)
				{
					temp=(TextBlock*)m_TextList.ItemAt(b);
					temp->m_Dirty=true;
				}
		}
	}
	
	
}

void	HtmlSyntaxColorer::TextDeleted(int Start,int Stop,const char* Text)
{
	TextBlock*	temp;
	ReparseAll(Text);

	int	textlength=m_TextList.CountItems();

	for(int i=0;i<textlength;i++)
	{	
		temp=(TextBlock*)m_TextList.ItemAt(i);
		temp->m_Dirty=false;
	}

	for(int i=0;i<textlength;i++)
	{	
		temp=(TextBlock*)m_TextList.ItemAt(i);
		if(Start>=temp->m_Start&&Stop<=temp->m_Stop)
		{
			temp->m_Dirty=true;

			if(i<textlength-2)
			{
				temp=(TextBlock*)m_TextList.ItemAt(i+1);
				temp->m_Dirty=true;
				temp=(TextBlock*)m_TextList.ItemAt(i+2);
				temp->m_Dirty=true;
			}
			
		}
	}
}

